// Styles
import '../../../src/components/VSlideGroup/VSlideGroup.sass';
// Components
import VIcon from '../VIcon';
import { VFadeTransition } from '../transitions';
// Extensions
import { BaseItemGroup } from '../VItemGroup/VItemGroup';
// Directives
import Resize from '../../directives/resize';
import Touch from '../../directives/touch';
// Utilities
import mixins from '../../util/mixins';
export var BaseSlideGroup = mixins(BaseItemGroup
/* @vue/component */
).extend({
    name: 'base-slide-group',
    directives: {
        Resize: Resize,
        Touch: Touch
    },
    props: {
        activeClass: {
            type: String,
            default: 'v-slide-item--active'
        },
        appendIcon: {
            type: String,
            default: '$vuetify.icons.next'
        },
        mobileBreakPoint: {
            type: [Number, String],
            default: 1264,
            validator: function validator(v) {
                return !isNaN(parseInt(v));
            }
        },
        prependIcon: {
            type: String,
            default: '$vuetify.icons.prev'
        },
        showArrows: Boolean
    },
    data: function data() {
        return {
            isOverflowing: false,
            resizeTimeout: 0,
            startX: 0,
            scrollOffset: 0,
            widths: {
                content: 0,
                wrapper: 0
            }
        };
    },
    computed: {
        __cachedAppend: function __cachedAppend() {
            return this.genTransition('append');
        },
        __cachedPrepend: function __cachedPrepend() {
            return this.genTransition('prepend');
        },
        classes: function classes() {
            return BaseItemGroup.options.computed.classes.call(this);
        },
        hasAffixes: function hasAffixes() {
            return (this.showArrows || !this.isMobile) && this.isOverflowing;
        },
        hasAppend: function hasAppend() {
            if (!this.hasAffixes) return false;
            var _widths = this.widths,
                content = _widths.content,
                wrapper = _widths.wrapper;
            // Check one scroll ahead to know the width of right-most item

            return content > this.scrollOffset + wrapper;
        },
        hasPrepend: function hasPrepend() {
            return this.hasAffixes && this.scrollOffset > 0;
        },
        isMobile: function isMobile() {
            return this.$vuetify.breakpoint.width < this.mobileBreakPoint;
        }
    },
    watch: {
        scrollOffset: function scrollOffset(val) {
            this.$refs.content.style.transform = 'translateX(' + -val + 'px)';
        },

        widths: 'setOverflow'
    },
    methods: {
        genAppend: function genAppend() {
            var _this = this;

            if (!this.isOverflowing && !this.showArrows) return null;
            var slot = this.$scopedSlots.append ? this.$scopedSlots.append({}) : this.$slots.append || this.__cachedAppend;
            return this.$createElement('div', {
                staticClass: 'v-slide-group__append',
                class: {
                    'v-slide-group__append--disabled': !this.hasAppend
                },
                on: {
                    click: function click() {
                        return _this.onAffixClick('append');
                    }
                },
                key: 'append'
            }, [slot]);
        },
        genContent: function genContent() {
            return this.$createElement('div', {
                staticClass: 'v-slide-group__content',
                ref: 'content'
            }, this.$slots.default);
        },
        genIcon: function genIcon(location) {
            var upperLocation = '' + location[0].toUpperCase() + location.slice(1);
            var hasAffix = this['has' + upperLocation];
            if (!this.showArrows && !hasAffix) return null;
            return this.$createElement(VIcon, {
                props: {
                    disabled: !hasAffix
                }
            }, this[location + 'Icon']);
        },
        genPrepend: function genPrepend() {
            var _this2 = this;

            if (!this.isOverflowing && !this.showArrows) return null;
            var slot = this.$scopedSlots.prepend ? this.$scopedSlots.prepend({}) : this.$slots.prepend || this.__cachedPrepend;
            return this.$createElement('div', {
                staticClass: 'v-slide-group__prepend',
                class: {
                    'v-slide-group__prepend--disabled': !this.hasPrepend
                },
                on: {
                    click: function click() {
                        return _this2.onAffixClick('prepend');
                    }
                },
                key: 'prepend'
            }, [slot]);
        },
        genTransition: function genTransition(location) {
            return this.$createElement(VFadeTransition, [this.genIcon(location)]);
        },
        genWrapper: function genWrapper() {
            var _this3 = this;

            return this.$createElement('div', {
                staticClass: 'v-slide-group__wrapper',
                directives: [{
                    name: 'touch',
                    value: {
                        start: function start(e) {
                            return _this3.overflowCheck(e, _this3.onTouchStart);
                        },
                        move: function move(e) {
                            return _this3.overflowCheck(e, _this3.onTouchMove);
                        },
                        end: function end(e) {
                            return _this3.overflowCheck(e, _this3.onTouchEnd);
                        }
                    }
                }],
                ref: 'wrapper'
            }, [this.genContent()]);
        },
        newOffset /* istanbul ignore next */: function newOffset(direction) {
            // Force reflow
            var clientWidth = this.$refs.wrapper.clientWidth;
            if (direction === 'prepend') {
                return Math.max(this.scrollOffset - clientWidth, 0);
            }
            return Math.min(this.scrollOffset + clientWidth, this.$refs.content.clientWidth - clientWidth);
        },
        onAffixClick: function onAffixClick(location) {
            this.$emit('click:' + location);
            this.scrollTo(location);
        },
        onResize: function onResize() {
            /* istanbul ignore next */
            if (this._isDestroyed) return;
            this.setWidths();
        },
        onTouchStart: function onTouchStart(e) {
            var content = this.$refs.content;

            this.startX = this.scrollOffset + e.touchstartX;
            content.style.setProperty('transition', 'none');
            content.style.setProperty('willChange', 'transform');
        },
        onTouchMove: function onTouchMove(e) {
            this.scrollOffset = this.startX - e.touchmoveX;
        },
        onTouchEnd: function onTouchEnd() {
            var _$refs = this.$refs,
                content = _$refs.content,
                wrapper = _$refs.wrapper;

            var maxScrollOffset = content.clientWidth - wrapper.clientWidth;
            content.style.setProperty('transition', null);
            content.style.setProperty('willChange', null);
            /* istanbul ignore else */
            if (this.scrollOffset < 0 || !this.isOverflowing) {
                this.scrollOffset = 0;
            } else if (this.scrollOffset >= maxScrollOffset) {
                this.scrollOffset = maxScrollOffset;
            }
        },
        overflowCheck: function overflowCheck(e, fn) {
            this.isOverflowing && fn(e);
        },
        scrollTo /* istanbul ignore next */: function scrollTo(location) {
            this.scrollOffset = this.newOffset(location);
        },
        setOverflow: function setOverflow() {
            this.isOverflowing = this.widths.wrapper < this.widths.content;
        },
        setWidths: function setWidths() {
            var _$refs2 = this.$refs,
                content = _$refs2.content,
                wrapper = _$refs2.wrapper;

            this.widths = {
                content: content ? content.clientWidth : 0,
                wrapper: wrapper ? wrapper.clientWidth : 0
            };
        }
    },
    render: function render(h) {
        return h('div', {
            staticClass: 'v-item-group v-slide-group',
            class: this.classes,
            directives: [{
                name: 'resize',
                value: this.onResize
            }]
        }, [this.genPrepend(), this.genWrapper(), this.genAppend()]);
    }
});
export default BaseSlideGroup.extend({
    name: 'v-slide-group',
    provide: function provide() {
        return {
            slideGroup: this
        };
    }
});
//# sourceMappingURL=VSlideGroup.js.map